import asyncio
import logging
from typing import Dict, List, Optional, Any
from datetime import datetime, timezone

from ..clients.hyperliquid_client import HyperliquidClient
from ..notifications.notification_manager import NotificationManager

logger = logging.getLogger(__name__)

class PositionTracker:
    """
    Simplified position tracker that mirrors exchange state.
    Monitors for position changes and saves stats when positions close.
    """
    
    def __init__(self, hl_client: HyperliquidClient, notification_manager: NotificationManager):
        self.hl_client = hl_client
        self.notification_manager = notification_manager
        self.trading_stats = None  # Will be lazy loaded
        
        # Track current positions
        self.current_positions: Dict[str, Dict] = {}
        self.is_running = False
        
    async def start(self):
        """Start position tracking"""
        if self.is_running:
            return
            
        self.is_running = True
        logger.info("Starting position tracker")
        
        # Initialize current positions
        await self._update_current_positions()
        
        # Start monitoring loop
        asyncio.create_task(self._monitoring_loop())
        
    async def stop(self):
        """Stop position tracking"""
        self.is_running = False
        logger.info("Stopping position tracker")
        
    async def _monitoring_loop(self):
        """Main monitoring loop"""
        while self.is_running:
            try:
                await self._check_position_changes()
                await asyncio.sleep(2)  # Check every 2 seconds
            except Exception as e:
                logger.error(f"Error in position tracking loop: {e}")
                await asyncio.sleep(5)
                
    async def _check_position_changes(self):
        """Check for any position changes"""
        try:
            previous_positions = self.current_positions.copy()
            await self._update_current_positions()
            
            # Compare with previous positions
            await self._process_position_changes(previous_positions, self.current_positions)
            
        except Exception as e:
            logger.error(f"Error checking position changes: {e}")
            
    async def _update_current_positions(self):
        """Update current positions from exchange"""
        try:
            positions = self.hl_client.get_positions()
            if not positions:
                return
                
            new_positions = {}
            for position in positions:
                size = float(position.get('szi', '0'))
                if size != 0:  # Only include open positions
                    symbol = position.get('coin', '')
                    if symbol:
                        new_positions[symbol] = {
                            'size': size,
                            'entry_px': float(position.get('entryPx', '0')),
                            'unrealized_pnl': float(position.get('unrealizedPnl', '0')),
                            'margin_used': float(position.get('marginUsed', '0')),
                            'max_leverage': float(position.get('maxLeverage', '1')),
                            'return_on_equity': float(position.get('returnOnEquity', '0'))
                        }
            
            self.current_positions = new_positions
            
        except Exception as e:
            logger.error(f"Error updating current positions: {e}")
            
    async def _process_position_changes(self, previous: Dict, current: Dict):
        """Process changes between previous and current positions"""
        
        # Find new positions
        for symbol in current:
            if symbol not in previous:
                await self._handle_position_opened(symbol, current[symbol])
                
        # Find closed positions  
        for symbol in previous:
            if symbol not in current:
                await self._handle_position_closed(symbol, previous[symbol])
                
        # Find changed positions
        for symbol in current:
            if symbol in previous:
                await self._handle_position_changed(symbol, previous[symbol], current[symbol])
                
    async def _handle_position_opened(self, symbol: str, position: Dict):
        """Handle new position opened"""
        try:
            size = position['size']
            side = "Long" if size > 0 else "Short"
            
            message = (
                f"🟢 Position Opened\n"
                f"Token: {symbol}\n"
                f"Side: {side}\n"
                f"Size: {abs(size):.4f}\n"
                f"Entry: ${position['entry_px']:.4f}\n"
                f"Leverage: {position['max_leverage']:.1f}x"
            )
            
            await self.notification_manager.send_notification(message)
            logger.info(f"Position opened: {symbol} {side} {abs(size)}")
            
        except Exception as e:
            logger.error(f"Error handling position opened for {symbol}: {e}")
            
    async def _handle_position_closed(self, symbol: str, position: Dict):
        """Handle position closed - save stats to database"""
        try:
            # Get current market price for PnL calculation
            market_data = self.hl_client.get_market_data(symbol)
            if not market_data:
                logger.error(f"Could not get market data for {symbol}")
                return
                
            current_price = float(market_data.get('markPx', 0))
            entry_price = position['entry_px']
            size = abs(position['size'])
            side = "Long" if position['size'] > 0 else "Short"
            
            # Calculate PnL
            if side == "Long":
                pnl = (current_price - entry_price) * size
            else:
                pnl = (entry_price - current_price) * size
                
            # Save to database
            await self._save_position_stats(symbol, side, size, entry_price, current_price, pnl)
            
            # Send notification
            pnl_emoji = "🟢" if pnl >= 0 else "🔴"
            message = (
                f"{pnl_emoji} Position Closed\n"
                f"Token: {symbol}\n"
                f"Side: {side}\n"
                f"Size: {size:.4f}\n"
                f"Entry: ${entry_price:.4f}\n"
                f"Exit: ${current_price:.4f}\n"
                f"PnL: ${pnl:.3f}"
            )
            
            await self.notification_manager.send_notification(message)
            logger.info(f"Position closed: {symbol} {side} PnL: ${pnl:.3f}")
            
        except Exception as e:
            logger.error(f"Error handling position closed for {symbol}: {e}")
            
    async def _handle_position_changed(self, symbol: str, previous: Dict, current: Dict):
        """Handle position size or direction changes"""
        try:
            prev_size = previous['size']
            curr_size = current['size']
            
            # Check if position reversed (long to short or vice versa)
            if (prev_size > 0 and curr_size < 0) or (prev_size < 0 and curr_size > 0):
                # Position reversed - close old, open new
                await self._handle_position_closed(symbol, previous)
                await self._handle_position_opened(symbol, current)
                return
                
            # Check if position size changed significantly
            size_change = abs(curr_size) - abs(prev_size)
            if abs(size_change) > 0.001:  # Threshold to avoid noise
                
                change_type = "Increased" if size_change > 0 else "Decreased"
                side = "Long" if curr_size > 0 else "Short"
                
                message = (
                    f"🔄 Position {change_type}\n"
                    f"Token: {symbol}\n"
                    f"Side: {side}\n"
                    f"New Size: {abs(curr_size):.4f}\n"
                    f"Change: {'+' if size_change > 0 else ''}{size_change:.4f}"
                )
                
                await self.notification_manager.send_notification(message)
                logger.info(f"Position changed: {symbol} {change_type} by {size_change:.4f}")
                
        except Exception as e:
            logger.error(f"Error handling position change for {symbol}: {e}")
            
    async def _save_position_stats(self, symbol: str, side: str, size: float, 
                                 entry_price: float, exit_price: float, pnl: float):
        """Save position statistics to database using existing TradingStats interface"""
        try:
            # Lazy load TradingStats to avoid circular imports
            if self.trading_stats is None:
                from ..stats.trading_stats import TradingStats
                self.trading_stats = TradingStats()
            
            # Use the existing process_trade_complete_cycle method
            lifecycle_id = self.trading_stats.process_trade_complete_cycle(
                symbol=symbol,
                side=side.lower(),
                entry_price=entry_price,
                exit_price=exit_price,
                amount=size,
                timestamp=datetime.now(timezone.utc).isoformat()
            )
            
            logger.info(f"Saved stats for {symbol}: PnL ${pnl:.3f}, lifecycle_id: {lifecycle_id}")
            
        except Exception as e:
            logger.error(f"Error saving position stats for {symbol}: {e}") 